#include <iostream>
#include "allegro.h"

#include "../shared/map/map.h"

const int MAP_SCROLL_THRES = 50;
const int MAP_SCROLL_SPEED = 5;


using namespace std;

volatile int target_frames = 0;
volatile int frames = 0;
volatile int fps = 0;
  
void ticker()
{
	
	target_frames++;
} END_OF_FUNCTION(ticker);

void fpsproc()
{
	fps = frames;
	frames = 0;
} END_OF_FUNCTION(fpsproc);
  
void init();


void grey(BITMAP *bm)
{
 for(int x = 0; x < bm->w; x++)
	for(int y = 0; y < bm->h; y++)
	{
		int color = _getpixel16(bm, x, y);

		int r = getr16(color);
		int g = getg16(color);
		int b = getb16(color);

		
		int c = (r + g + b) / 3;

		if(b > 100)
			_putpixel16(bm, x, y, makecol16(c, c, c - 100));
		else
			_putpixel16(bm, x, y, makecol16(c, c, c));
	}
}



  

int main(int argc, char *argv[])
{
	// Init all allegro stuff:
	init();

	gui_fg_color = makecol(0, 0, 0);
	gui_bg_color = makecol(239, 220, 131);
	
	// Create double buffer:
	BITMAP *buffer = create_bitmap(SCREEN_W, SCREEN_H);
	

	// Load the selector:
	BITMAP *selector = load_bitmap("images/selector.pcx", NULL);
	if (!selector) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Error reading images/selector.pcx!\n");
		return 1;
	}
	
	// Create map:
	map *current_map;
	current_map = new map();
	current_map->layer_create(100, 100, 0, 255);	
	current_map->layer_create(100, 100, 0, 255);	

	for(int x = 0; x < 100; x++)
		for(int y = 0; y < 100; y++)
			current_map->set(0, x, y, 2);
	
	// Load tileset:
	tileset *current_ts = new tileset;
	current_ts->load("images/tileset.pcx");
	
	int y, x, tile;
	y = x = tile = 0;
	
	int camera_x = 0;
	int camera_y = 0;

	int layernr = 0;

	bool editor_on = true;
			
	while(editor_on)
	{

		//blit(bg, buffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		
		int real_x = mouse_x / TILE_W + camera_x / TILE_W;
		int real_y = mouse_y / TILE_H + camera_y / TILE_H;		
		
		int actual_frames = target_frames;
		target_frames = 0;
        
		for( ; actual_frames > 0 ; actual_frames--)
		{	
			
			if(keypressed())
			{


				if(key[KEY_UP])
				{
					if(tile > 0)tile--;
				}
				if(key[KEY_DOWN])
				{
					if(tile < current_ts->count() - 1)tile++;			
				}
				if(key[KEY_LEFT])
				{
					if(layernr > 0)
						layernr--;
				}
				if(key[KEY_RIGHT])
				{
					if(layernr < current_map->layer_count() - 1)
						layernr++;
				}
				if(key[KEY_S] || key[KEY_L] || key[KEY_ESC])
				{
					remove_int(ticker); remove_int(fpsproc);
					grey(buffer);
					blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

					if(key[KEY_ESC])
					{
						if(alert("Do you really want to", "quit the map editor?", "", "Yes", "No", KEY_Y, KEY_N) == 1)
						{
							editor_on = false;
						}

					}

					if(key[KEY_S])
					{
						
						char filename[500];
						strcpy(filename, "./maps/");

						if(file_select_ex("Save map as...", filename, "map;/-h", 500, 500, 400))
							current_map->save(filename);
					}
				
					if(key[KEY_L])
					{
						char filename[500];
						strcpy(filename, "./maps/");

						if(file_select_ex("Load map...", filename, "map;/-h", 500, 500, 350))
							current_map->load(filename);
					}

					install_int_ex(ticker, BPS_TO_TIMER(60));
					install_int_ex(fpsproc, BPS_TO_TIMER(1));

				}
				clear_keybuf();
			}
			
			if(mouse_b & 1)
			{
				current_map->set(layernr, real_x, real_y, tile);
			}
			
			if(mouse_b & 2)
			{
				
			}
			
			
			if(mouse_x > SCREEN_W - MAP_SCROLL_THRES)
				camera_x += MAP_SCROLL_SPEED;
			if(mouse_y > SCREEN_H - MAP_SCROLL_THRES)
				camera_y += MAP_SCROLL_SPEED;
			if(mouse_x < MAP_SCROLL_THRES)
				if(camera_x > 0)camera_x -= MAP_SCROLL_SPEED;
			if(mouse_y < MAP_SCROLL_THRES)
				if(camera_y > 0)camera_y -= MAP_SCROLL_SPEED;
				
		}

		clear_bitmap(buffer);
		
		current_map->render(buffer, current_ts, camera_x, camera_y);
		
		rectfill(buffer, SCREEN_W - TILE_W - 20, 0, SCREEN_W, SCREEN_H, makecol(0, 0, 0));	

		for(int i = 0; i < SCREEN_H / (TILE_H + 5) ; i++)
		{
			if(i + tile >= 0 && i + tile < current_ts->count())
			{
				blit(current_ts->tile[i + tile], buffer, 0, 0, SCREEN_W - TILE_W - 10, i * (TILE_H + 10) + 10, TILE_W, TILE_H);
			}
			
		}
		rect(buffer, SCREEN_W - TILE_W - 11, 9, SCREEN_W - 10, TILE_H + 10, makecol(255, 0, 0));
		
		textprintf_ex(buffer, font, 10, 10, makecol(255, 255, 255), 0, "FPS: %i", fps);
		textprintf_ex(buffer, font, 10, 20, makecol(255, 255, 255), 0, "L: %i X: %i, Y: %i, Tile: %i", layernr, real_x, real_y, tile);
		textprintf_ex(buffer, font, 10, 40, makecol(255, 255, 255), 0, "Tile at this point is: %i", current_map->get(layernr, real_x, real_y));
				
		masked_blit(selector, buffer, 0, 0, mouse_x / TILE_W * TILE_W - camera_x % TILE_W, mouse_y / TILE_H * TILE_H  - camera_y % TILE_H, TILE_W, TILE_H);
				
		
		//vsync();
		blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
		
		
		
		//readkey();
		frames++;
		
	}	
	
	delete current_map;
	delete current_ts;
	
	return 0;	
	
}
END_OF_MAIN();

void init()
{
	allegro_init();
	
	install_keyboard();
	install_mouse();
	install_timer();

	set_color_depth(16);

	if(set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0) != 0)
	{
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Error setting 16 bit graphics mode\n%s\n", allegro_error);
		exit(1);
	}	
	
	set_color_conversion(COLORCONV_TOTAL);
	
	LOCK_VARIABLE(target_frames);
	LOCK_VARIABLE(frames);
	LOCK_VARIABLE(fps);
	LOCK_FUNCTION(ticker);
	LOCK_FUNCTION(fpsproc);
		
	install_int_ex(ticker, BPS_TO_TIMER(60));
	install_int_ex(fpsproc, BPS_TO_TIMER(1));
		
}
